"""
This module contains unit tests for the SingleStatementParser and DeclarationsTreeVisitor.
It verifies that the parser correctly interprets and parses single statement types,
and the visitor accurately transforms the parsed AST into the expected string representation.

This feature is tested very rigorously as any UPPAAL automaton fundamentally relies on
guards, updates and synchronizations on its transitions.
"""
from uppaal2jetracer.parser.declarationparser.declaration_parser import SingleStatementParser
from uppaal2jetracer.declarations.declarations_tree_visitor import DeclarationsTreeVisitor
from uppaal2jetracer.parser.declarationparser.plyparser import ParseError

ss_parser = SingleStatementParser(lex_optimize = False, yacc_optimize = False, yacc_debug = False)
tree_visitor = DeclarationsTreeVisitor()


def test_semicolon_fail():
    """
    The test defines a declaration statement with a semicolon at its end.

    :raises ParseError: if the parser correctly parses the statement
    """
    declaration_code = "x;"
    try:
        ss_parser.parse(declaration_code, "", False)
        assert False
    except ParseError:
        assert True


# Test variable names

def test_one_letter_name():
    """
    Tests the parsing of a variable consisting of a single letter.

    Parses and evaluates the statement "x", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement = "x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(ID: x)"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_multi_letter_name():
    """
    Tests the parsing of a variable consisting of multiple letters.

    Parses and evaluates the statement "abc", "Abc" and "ABC", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement_1 = "abc"
    statement_2 = "Abc"
    statement_3 = "ABC"
    declaration_ast_1 = ss_parser.parse(statement_1, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    declaration_ast_3 = ss_parser.parse(statement_3, "", False)
    assert (("(FileAST: "
             "(ID: abc)"
             ")") == tree_visitor.visit_fileast(declaration_ast_1)
            and
            ("(FileAST: "
             "(ID: Abc)"
             ")") == tree_visitor.visit_fileast(declaration_ast_2)
            and
            ("(FileAST: "
             "(ID: ABC)"
             ")") == tree_visitor.visit_fileast(declaration_ast_3)
            )


def test_underscore_name():
    """
    Tests the parsing of a variable with an underscore.

    Parses and evaluates the statement "x_y", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement = "x_y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(ID: x_y)"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_number_name():
    """
    Tests the parsing of a variable with numbers.

    Parses and evaluates the statement "x1", "xyz123" and "x1y2z3a", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement_1 = "x1"
    statement_2 = "xyz123"
    statement_3 = "x1y2z3a"
    declaration_ast_1 = ss_parser.parse(statement_1, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    declaration_ast_3 = ss_parser.parse(statement_3, "", False)
    assert (("(FileAST: "
             "(ID: x1)"
             ")") == tree_visitor.visit_fileast(declaration_ast_1)
            and
            ("(FileAST: "
             "(ID: xyz123)"
             ")") == tree_visitor.visit_fileast(declaration_ast_2)
            and
            ("(FileAST: "
             "(ID: x1y2z3a)"
             ")") == tree_visitor.visit_fileast(declaration_ast_3)
            )


def test_number_name_fail():
    """
    The test defines a declaration statement with a variable that has a number at the beginning.

    :raises ParseError: if the parser correctly parses the statement
    """
    declaration_code = "0x"
    try:
        ss_parser.parse(declaration_code, "", False)
        assert False
    except ParseError:
        assert True


# Test assignment of general datatypes

def test_x_bool_assignment():
    """
    Tests the assignment of boolean values to variable x.

    Parses and evaluates the statements "x = true" and "x = false", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement_true = "x = true"
    statement_false = "x = false"
    declaration_ast_true = ss_parser.parse(statement_true, "", False)
    declaration_ast_false = ss_parser.parse(statement_false, "", False)
    assert (("(FileAST: "
             "(Assignment: (ID: x) = ((Constant: true)))"
             ")") == tree_visitor.visit_fileast(declaration_ast_true)
            and
            ("(FileAST: "
             "(Assignment: (ID: x) = ((Constant: false)))"
             ")") == tree_visitor.visit_fileast(declaration_ast_false)
            )


def test_x_integer_assignment():
    """
    Tests the assignment of integer values to variable x.

    Parses and evaluates the statements "x = 999999999999", "x = 1", "x = 0", and "x = -1",
    then verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement_big = "x = 999999999999"
    statement_pos = "x = 1"
    statement_zero = "x = 0"
    statement_neg = "x = -1"
    declaration_ast_big = ss_parser.parse(statement_big, "", False)
    declaration_ast_pos = ss_parser.parse(statement_pos, "", False)
    declaration_ast_zero = ss_parser.parse(statement_zero, "", False)
    declaration_ast_neg = ss_parser.parse(statement_neg, "", False)
    assert (("(FileAST: "
             "(Assignment: (ID: x) = ((Constant: 999999999999)))"
             ")") == tree_visitor.visit_fileast(declaration_ast_big)
            and ("(FileAST: "
                 "(Assignment: (ID: x) = ((Constant: 1)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_pos)
            and ("(FileAST: "
                 "(Assignment: (ID: x) = ((Constant: 0)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_zero)
            and ("(FileAST: "
                 "(Assignment: (ID: x) = ((UnaryOp: -(Constant: 1))))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_neg)
            )


def test_x_double_assignment():
    """
    Tests the assignment of floating-point values to variable x.

    Parses and evaluates the statements "x = 3.14", "x = 0.0", and "x = -3.14", then
    verifies that the resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement_pos = "x = 3.14"
    statement_zero = "x = 0.0"
    statement_neg = "x = -3.14"
    declaration_ast_pos = ss_parser.parse(statement_pos, "", False)
    declaration_ast_zero = ss_parser.parse(statement_zero, "", False)
    declaration_ast_neg = ss_parser.parse(statement_neg, "", False)
    assert (("(FileAST: "
             "(Assignment: (ID: x) = ((Constant: 3.14)))"
             ")") == tree_visitor.visit_fileast(declaration_ast_pos)
            and ("(FileAST: "
                 "(Assignment: (ID: x) = ((Constant: 0.0)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_zero)
            and ("(FileAST: "
                 "(Assignment: (ID: x) = ((UnaryOp: -(Constant: 3.14))))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_neg)
            )


def test_unary_negation():
    """
    Tests unary negation of variable x.

    Parses and evaluates the statement "!x", then verifies that the resulting AST
    matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement = "!x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(ID: x))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_double_unary_negation():
    """
    Tests unary negation of variable x.

    Parses and evaluates the statement "!!x", then verifies that the resulting AST
    matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement = "!!x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(UnaryOp: !(ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_eq_y_plus_3():
    """
    Tests assignment of an arithmetic expression to variable x.

    Parses and evaluates the statement "x = y + 3", then verifies that the
    resulting AST matches the expected structure.

    :raises AssertionError: If the AST does not match the expected structure.
    """
    statement = "x = y + 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(Assignment: (ID: x) = ((BinaryOp: ((ID: y)) + ((Constant: 3)))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


# Test >

def test_x_gt_y():
    """
    Tests the parsing of the expression "x > y".

    This test verifies that the parser correctly generates an AST for
    the greater-than comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x > y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) > ((ID: y)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_gt_3():
    """
    Tests the parsing of the expression "x > 3".

    This test ensures that the parser correctly handles a greater-than
    comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x > 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) > ((Constant: 3)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_gt_x():
    """
    Tests the parsing of the expression "3 > x".

    This test verifies that the parser correctly represents a greater-than
    comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 > x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) > ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_gt_5():
    """
    Tests the parsing of the expression "3 > 5".

    This test checks whether the parser correctly represents a comparison
    between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 > 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) > ((Constant: 5)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_ngt_y():
    """
    Tests the parsing of the expression "!(x > y)".

    This test ensures that the parser correctly handles the negation
    of a greater-than comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x > y)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) > ((ID: y))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_ngt_3():
    """
    Tests the parsing of the expression "!(x > 3)".

    This test verifies that the parser correctly represents the negation
    of a greater-than comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x > 3)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) > ((Constant: 3))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_ngt_x():
    """
    Tests the parsing of the expression "!(3 > x)".

    This test ensures that the parser correctly handles the negation of a
    greater-than comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 > x)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) > ((ID: x))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_ngt_5():
    """
    Tests the parsing of the expression "!(3 > 5)".

    This test verifies that the parser correctly represents the negation
    of a greater-than comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 > 5)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) > ((Constant: 5))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


# Test >=

def test_x_ge_y():
    """
    Tests the parsing of the expression "x >= y".

    This test verifies that the parser correctly generates an AST for
    the greater-than-or-equal comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x >= y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) >= ((ID: y)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_ge_3():
    """
    Tests the parsing of the expression "x >= 3".

    This test ensures that the parser correctly handles a greater-than-or-equal
    comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x >= 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) >= ((Constant: 3)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_ge_x():
    """
    Tests the parsing of the expression "3 >= x".

    This test verifies that the parser correctly represents a greater-than-or-equal
    comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 >= x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) >= ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_ge_5():
    """
    Tests the parsing of the expression "3 >= 5".

    This test checks whether the parser correctly represents a comparison
    between two constant integers using the greater-than-or-equal operator.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 >= 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) >= ((Constant: 5)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nge_y():
    """
    Tests the parsing of the expression "!(x >= y)".

    This test ensures that the parser correctly handles the negation
    of a greater-than-or-equal comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x >= y)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) >= ((ID: y))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nge_3():
    """
    Tests the parsing of the expression "!(x >= 3)".

    This test verifies that the parser correctly represents the negation
    of a greater-than-or-equal comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x >= 3)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) >= ((Constant: 3))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nge_x():
    """
    Tests the parsing of the expression "!(3 >= x)".

    This test ensures that the parser correctly handles the negation of a
    greater-than-or-equal comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 >= x)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) >= ((ID: x))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nge_5():
    """
    Tests the parsing of the expression "!(3 >= 5)".

    This test verifies that the parser correctly represents the negation
    of a greater-than-or-equal comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 >= 5)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) >= ((Constant: 5))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


# Test ==

def test_x_eq_y():
    """
    Tests the parsing of the expression "x == y".

    This test verifies that the parser correctly generates an AST for
    an equality comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x == y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) == ((ID: y)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_eq_3():
    """
    Tests the parsing of the expression "x == 3".

    This test ensures that the parser correctly handles an equality
    comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x == 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) == ((Constant: 3)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_eq_x():
    """
    Tests the parsing of the expression "3 == x".

    This test verifies that the parser correctly represents an equality
    comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 == x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) == ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_eq_5():
    """
    Tests the parsing of the expression "3 == 5".

    This test checks whether the parser correctly represents an equality
    comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 == 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) == ((Constant: 5)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_neq_y():
    """
    Tests the parsing of the expressions "!(x == y)" and "x != y".

    This test ensures that the parser correctly represents both
    the explicit negation of an equality comparison and the "!=" operator.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x == y)"
    statement_2 = "x != y"
    declaration_ast = ss_parser.parse(statement, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    assert (("(FileAST: "
             "(UnaryOp: !(BinaryOp: ((ID: x)) == ((ID: y))))"
             ")") == tree_visitor.visit_fileast(declaration_ast)
            and ("(FileAST: "
                 "(BinaryOp: ((ID: x)) != ((ID: y)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_2)
            )


def test_x_neq_3():
    """
    Tests the parsing of the expressions "!(x == 3)" and "x != 3".

    This test verifies that the parser correctly represents both
    the explicit negation of an equality comparison and the "!=" operator
    when comparing an identifier with a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x == 3)"
    statement_2 = "x != 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    assert (("(FileAST: "
             "(UnaryOp: !(BinaryOp: ((ID: x)) == ((Constant: 3))))"
             ")") == tree_visitor.visit_fileast(declaration_ast)
            and ("(FileAST: "
                 "(BinaryOp: ((ID: x)) != ((Constant: 3)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_2)
            )


def test_3_neq_x():
    """
    Tests the parsing of the expressions "!(3 == x)" and "3 != x".

    This test ensures that the parser correctly handles both
    the explicit negation of an equality comparison and the "!=" operator
    when a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 == x)"
    statement_2 = "3 != x"
    declaration_ast = ss_parser.parse(statement, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    assert (("(FileAST: "
             "(UnaryOp: !(BinaryOp: ((Constant: 3)) == ((ID: x))))"
             ")") == tree_visitor.visit_fileast(declaration_ast)
            and ("(FileAST: "
                 "(BinaryOp: ((Constant: 3)) != ((ID: x)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_2)
            )


def test_3_neq_5():
    """
    Tests the parsing of the expressions "!(3 == 5)" and "3 != 5".

    This test verifies that the parser correctly represents both
    the explicit negation of an equality comparison and the "!=" operator
    when comparing two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 == 5)"
    statement_2 = "3 != 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    declaration_ast_2 = ss_parser.parse(statement_2, "", False)
    assert (("(FileAST: "
             "(UnaryOp: !(BinaryOp: ((Constant: 3)) == ((Constant: 5))))"
             ")") == tree_visitor.visit_fileast(declaration_ast)
            and ("(FileAST: "
                 "(BinaryOp: ((Constant: 3)) != ((Constant: 5)))"
                 ")") == tree_visitor.visit_fileast(declaration_ast_2)
            )


# Test <=

def test_x_le_y():
    """
    Tests the parsing of the expression "x <= y".

    This test verifies that the parser correctly generates an AST for
    a less-than-or-equal comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x <= y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) <= ((ID: y)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_le_3():
    """
    Tests the parsing of the expression "x <= 3".

    This test ensures that the parser correctly handles a less-than-or-equal
    comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x <= 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) <= ((Constant: 3)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_le_x():
    """
    Tests the parsing of the expression "3 <= x".

    This test verifies that the parser correctly represents a less-than-or-equal
    comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 <= x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) <= ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_le_5():
    """
    Tests the parsing of the expression "3 <= 5".

    This test checks whether the parser correctly represents a less-than-or-equal
    comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 <= 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) <= ((Constant: 5)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nle_y():
    """
    Tests the parsing of the expression "!(x <= y)".

    This test ensures that the parser correctly represents the explicit negation
    of a less-than-or-equal comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x <= y)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) <= ((ID: y))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nle_3():
    """
    Tests the parsing of the expression "!(x <= 3)".

    This test verifies that the parser correctly represents the explicit negation
    of a less-than-or-equal comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x <= 3)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) <= ((Constant: 3))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nle_x():
    """
    Tests the parsing of the expression "!(3 <= x)".

    This test ensures that the parser correctly handles the explicit negation
    of a less-than-or-equal comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 <= x)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) <= ((ID: x))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nle_5():
    """
    Tests the parsing of the expression "!(3 <= 5)".

    This test verifies that the parser correctly represents the explicit negation
    of a less-than-or-equal comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 <= 5)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) <= ((Constant: 5))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


# Test <

def test_x_lt_y():
    """
    Tests the parsing of the expression "x < y".

    This test verifies that the parser correctly generates an AST for
    a less-than comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x < y"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) < ((ID: y)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_lt_3():
    """
    Tests the parsing of the expression "x < 3".

    This test ensures that the parser correctly handles a less-than
    comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x < 3"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) < ((Constant: 3)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_lt_x():
    """
    Tests the parsing of the expression "3 < x".

    This test verifies that the parser correctly represents a less-than
    comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 < x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) < ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_lt_5():
    """
    Tests the parsing of the expression "3 < 5".

    This test checks whether the parser correctly represents a less-than
    comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "3 < 5"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((Constant: 3)) < ((Constant: 5)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nlt_y():
    """
    Tests the parsing of the expression "!(x < y)".

    This test ensures that the parser correctly represents the explicit negation
    of a less-than comparison between two identifiers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x < y)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) < ((ID: y))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_nlt_3():
    """
    Tests the parsing of the expression "!(x < 3)".

    This test verifies that the parser correctly represents the explicit negation
    of a less-than comparison between an identifier and a constant integer.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(x < 3)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((ID: x)) < ((Constant: 3))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nlt_x():
    """
    Tests the parsing of the expression "!(3 < x)".

    This test ensures that the parser correctly handles the explicit negation
    of a less-than comparison where a constant integer is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 < x)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) < ((ID: x))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_3_nlt_5():
    """
    Tests the parsing of the expression "!(3 < 5)".

    This test verifies that the parser correctly represents the explicit negation
    of a less-than comparison between two constant integers.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!(3 < 5)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(BinaryOp: ((Constant: 3)) < ((Constant: 5))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


# Test funccalls

def test_neg_funccall():
    """
    Tests the parsing of the expression "!isgreaterthan(x, y)".

    This test verifies that the parser correctly represents the negation
    of a function call with two arguments.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "!isgreaterthan(x, y)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(UnaryOp: !(FuncCall: (ID: isgreaterthan)"
            "((ExprList: (ID: x), (ID: y)))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_funccall_assignment():
    """
    Tests the parsing of the expression "x = add(5, 6)".

    This test ensures that the parser correctly represents an assignment
    where a function call with two constant arguments is assigned to a variable.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x = add(5, 6)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(Assignment: (ID: x) = ((FuncCall: (ID: add)"
            "((ExprList: (Constant: 5), (Constant: 6))))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_x_eq_funccall():
    """
    Tests the parsing of the expression "x == add(5, 6)".

    This test verifies that the parser correctly represents an equality comparison
    between an identifier and a function call with two constant arguments.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "x == add(5, 6)"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((ID: x)) == ((FuncCall: (ID: add)"
            "((ExprList: (Constant: 5), (Constant: 6))))))"
            ")") == tree_visitor.visit_fileast(declaration_ast)


def test_funccall_eq_x():
    """
    Tests the parsing of the expression "add(5, 6) == x".

    This test ensures that the parser correctly represents an equality comparison
    where a function call with two constant arguments is compared to an identifier.

    :raises AssertionError: If the generated AST does not match the expected structure.
    """
    statement = "add(5, 6) == x"
    declaration_ast = ss_parser.parse(statement, "", False)
    assert ("(FileAST: "
            "(BinaryOp: ((FuncCall: (ID: add)((ExprList: (Constant: 5), (Constant: 6)))))"
            " == ((ID: x)))"
            ")") == tree_visitor.visit_fileast(declaration_ast)
