"""Command system test

This file provides tests for the command system.
"""

import os
from typing import Tuple

from uppaal2jetracer.cli import CLI
from uppaal2jetracer.command_system import CommandResultType, ParseCommand, QuitCommand, \
    VersionCommand, ProjectCommand, RunCommand
from uppaal2jetracer.versioncontrol.database import DatabaseConnection
from uppaal2jetracer.versioncontrol.versioncontrol import VersionManager, ProjectManager


def create_database() -> Tuple[ProjectManager, VersionManager]:
    """
    Create the test database.
    """

    db = DatabaseConnection("data" + os.sep + "database", "test_command_system")
    pm = ProjectManager(db)
    remove_database(pm)

    db = DatabaseConnection("data" + os.sep + "database", "test_command_system")
    pm = ProjectManager(db)
    vm = VersionManager(db)

    return pm, vm


def remove_database(pm: ProjectManager):
    """
    Remove the test database.
    """

    pm.db.session.close()
    pm.db.engine.dispose()

    pm.db.remove_db()


def test_parse_command():
    """
    Test that the parse command works as expected.
    """

    db = create_database()
    pm = db[0]
    vm = db[1]
    pm.add_project("test_project")

    parse_command = ParseCommand(vm)

    result = parse_command.execute([])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = parse_command.execute(["not_a_path.xml"])
    assert result.message == "'not_a_path.xml' is not a valid file."
    assert result.result_type == CommandResultType.FAILURE

    file_path = os.path.dirname(os.path.realpath(__file__)) + os.sep + "testAutomaton.pkl"
    result = parse_command.execute([file_path])
    assert result.message == f"'{file_path}' is not a valid xml file."
    assert result.result_type == CommandResultType.FAILURE

    file_path = os.path.dirname(os.path.realpath(__file__)) + os.sep + "testAutomaton.xml"
    result = parse_command.execute([file_path])
    assert result.message == (f"Parsed successfully and saved in "
                              f"'{file_path.replace(".xml", ".pkl")}'.")

    remove_database(pm)


def test_project_command():
    """
    Test that the project command works as expected.
    """

    db = create_database()
    pm = db[0]

    project_command = ProjectCommand(pm)

    result = project_command.execute([])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["not_an_arg", "not_a_param"])
    assert result.message == "'not_an_arg' is not a valid argument."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["current"])
    assert result.message == "No current project exists."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["new", "test_project_1"])
    assert result.message == "New project 'test_project_1' created."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["new", "test_project_2"])
    assert result.message == "New project 'test_project_2' created."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["new", "test_project_3"])
    assert result.message == "New project 'test_project_3' created."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["current"])
    assert result.message == "Current project is 'test_project_3'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["open", "test_project_2"])
    assert result.message == "Opened project 'test_project_2'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["current"])
    assert result.message == "Current project is 'test_project_2'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["list", "?"])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["list"])
    assert result.message == """1: test_project_1
2: test_project_2
3: test_project_3"""
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["config", "?"])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["config"])
    assert result.message == "Current version limit is 10."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["max", "?"])
    assert result.message == "'?' is not a valid argument."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["max", "-1"])
    assert result.message == "'-1' is not a valid argument."
    assert result.result_type == CommandResultType.FAILURE

    result = project_command.execute(["max", "8"])
    assert result.message == "Set maximum number of versions per project to 8."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["config"])
    assert result.message == "Current version limit is 8."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["delete", "test_project_1"])
    assert result.message == "Deleted project 'test_project_1'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["delete", "test_project_2"])
    assert result.message == "Deleted project 'test_project_2'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["delete", "test_project_3"])
    assert result.message == "Deleted project 'test_project_3'."
    assert result.result_type == CommandResultType.SUCCESS

    result = project_command.execute(["list"])
    assert result.message == ""
    assert result.result_type == CommandResultType.SUCCESS

    remove_database(pm)


def test_version_command():
    """
    Test that the version command works as expected.
    """

    db = create_database()
    pm = db[0]
    vm = db[1]
    cli: CLI = CLI(vm, pm)
    pm.add_project("test_project")
    parse_command = ParseCommand(vm)
    file_path = os.path.dirname(os.path.realpath(__file__)) + os.sep + "testAutomaton.xml"
    result = parse_command.execute([file_path])
    assert result.result_type == CommandResultType.SUCCESS

    version_command = VersionCommand(cli, vm)

    result = version_command.execute([])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = version_command.execute(["not_an_arg", "not_a_param"])
    assert result.message == "'not_an_arg' is not a valid argument."
    assert result.result_type == CommandResultType.FAILURE

    result = version_command.execute(["run", "100"])
    assert result.message == "No version can be found."
    assert result.result_type == CommandResultType.FAILURE

    result = version_command.execute(["list"])
    assert "1: testAutomaton: " in result.message
    assert result.result_type == CommandResultType.SUCCESS

    result = version_command.execute(["fav", "1"])
    assert result.message == "Version 1 is now a favorite."
    assert result.result_type == CommandResultType.SUCCESS

    result = version_command.execute(["fav", "1"])
    assert result.message == "Version 1 is now no longer a favorite."
    assert result.result_type == CommandResultType.SUCCESS

    result = version_command.execute(["delete", "1"])
    assert result.message == "Deleted version 1 from project."
    assert result.result_type == CommandResultType.SUCCESS

    remove_database(pm)


def test_run_command():
    """
    Test that the run command works as expected.
    """

    run_command = RunCommand()

    result = run_command.execute([])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    result = run_command.execute(["not_an_arg"])
    assert result.message == "'not_an_arg' is not a valid file."
    assert result.result_type == CommandResultType.FAILURE

    result = run_command.execute(["not_an_arg", "not_a_param"])
    assert result.message == "Invalid number of arguments provided."
    assert result.result_type == CommandResultType.FAILURE

    file_path = os.path.dirname(os.path.realpath(__file__)) + os.sep + "testAutomaton.xml"
    result = run_command.execute([file_path])
    assert result.message == f"'{file_path}' is not a valid pkl file."
    assert result.result_type == CommandResultType.FAILURE


def test_quit_command():
    """
    Test that the quit command works as expected.
    """

    db = create_database()
    pm = db[0]
    vm = db[1]

    cli: CLI = CLI(vm, pm)
    quit_command = QuitCommand(cli)

    result = quit_command.execute([])
    assert result.message == ""
    assert result.result_type == CommandResultType.SUCCESS

    remove_database(pm)
