"""
Test cases for the frame class.
"""
from uppaal2jetracer.declarations.declarations_ast import ID, Constant, Assignment, FuncDef
from uppaal2jetracer.uppaalmodel.elements import Update
from uppaal2jetracer.uppaalmodel.frame import Frame
from uppaal2jetracer.uppaalmodel.variable import IntVariable


def test_access_variable_frame():
    """
    Test the access to a variable in a frame.
    """
    f = Frame()
    v = IntVariable(False, 23)
    f.add_binding("var", v)

    r = f.get_variable("var")
    assert r == v


def test_access_variable_parent_frame():
    """
    Test the access to a variable in a parent frame.
    """
    f1 = Frame()
    f2 = Frame(f1)
    v = IntVariable(False, 23)
    f1.add_binding("var", v)

    r = f2.get_variable("var")
    assert r == v


def test_access_variable_in_lower_frame():
    """
    Test the access to a variable in a lower frame.
    """
    f1 = Frame()
    f2 = Frame(f1)
    v1 = IntVariable(False, 23)
    v2 = IntVariable(False, 23)
    f1.add_binding("var", v1)
    f2.add_binding("var", v2)

    r = f2.get_variable("var")
    assert r == v2


def test_no_variable_found():
    """
    Test what happens if no variable is found.
    """
    f = Frame()

    r = f.get_variable("var")
    assert r is None


def test_no_variable_in_parent_found():
    """
    Test if no variable in parent frame is found.
    """
    f1 = Frame()
    f2 = Frame(f1)

    r = f2.get_variable("var")
    assert r is None


def test_access_function_frame():
    """
    Test the access to a function in a frame.
    """
    f = Frame()
    n = FuncDef("name", None, None)
    f.add_function("node", n)

    r = f.get_function("node")
    assert r == n


def test_access_function_parent_frame():
    """
    Test the access to a function to a parent frame.
    """
    f1 = Frame()
    f2 = Frame(f1)
    n = FuncDef("name", None, None)
    f1.add_function("node", n)

    r = f2.get_function("node")
    assert r == n


def test_access_function_in_lower_frame():
    """
    Test the access to a function in a lower frame.
    """
    f1 = Frame()
    f2 = Frame(f1)
    n1 = FuncDef("name", None, None)
    n2 = FuncDef("name", None, None)
    f1.add_function("node", n1)
    f2.add_function("node", n2)

    r = f2.get_function("node")
    assert r == n2


def test_no_function_found():
    """
    Test what happens if no function is found.
    """
    f = Frame()

    r = f.get_function("node")
    assert r is None


def test_no_function_in_parent_found():
    """
    Test what happens if no function in parent is found.
    """
    f1 = Frame()
    f2 = Frame(f1)

    r = f2.get_function("node")
    assert r is None


def test_temporary_frame():
    """
    Test temporary frame function.
    """
    name = ID("x")
    value = Constant("int", 5)
    assignment = Assignment("=", name, value)
    frame = Frame()
    integer = IntVariable(False, 4)
    frame.add_binding("x", integer)
    update = Update([assignment])
    new_frame = frame.create_temporary_frame(update)

    assert frame.get_variable("x").value == 4
    assert new_frame.get_variable("x").value == 4

    frame.get_variable("x").value = 8

    assert frame.get_variable("x").value == 8
    assert new_frame.get_variable("x").value == 4

    new_frame.get_variable("x").value = 100

    assert frame.get_variable("x").value == 8
    assert new_frame.get_variable("x").value == 100
