"""
Test cases for the variable class.
"""
import pytest

from uppaal2jetracer.uppaalmodel.variable import (IntVariable, Array,
                                                  Channel, Range, BelowRangeError,
                                                  ExceedsRangeError, ImmutableValueError)
from uppaal2jetracer.uppaalmodel.elements import (Transition, Guard,
                                                  Update, Synchronization, Location, Invariant)
from uppaal2jetracer.declarations.declarations_ast import ID


def test_array_set_and_get():
    """
    Test the set and get of the array variable.
    """
    v1 = IntVariable(False, 34)
    v2 = IntVariable(False, 456)

    a = Array(False, [v1])

    assert a.get_index(0) == v1
    assert a.get_index(0) != v2

    a.set_index(0, v2)

    assert a.get_index(0) != v1
    assert a.get_index(0) == v2


def test_array_in_array_access():
    """
    Test what happens if you access an array in an array.
    """
    v1 = IntVariable(False, 34)
    v2 = IntVariable(False, 456)

    a1 = Array(False, [v1])
    a2 = Array(False, [a1])

    value = a2.get_index(0)
    if isinstance(value, Array):
        assert value.get_index(0) == v1
        assert value.get_index(0) != v2

        value.set_index(0, v2)

        assert value.get_index(0) != v1
        assert value.get_index(0) == v2


def test_channel_has_pair():
    """
    Test the has_pair method of the has pair method.
    """
    ch1 = Channel(False)
    ch2 = Channel(False)
    n = ID("name")
    g = Guard(n)
    u = Update([n])
    s = Synchronization(False, n)

    l1 = Location("id0", "location1")
    l2 = Location("id1", "location2")
    t = Transition(g, u, s)
    t.source = l1
    t.target = l2
    l1.add_transition(t)

    assert not ch1.has_pair()

    ch1.register_sending(t)
    ch2.register_receiving(t)

    assert not ch1.has_pair()
    assert not ch2.has_pair()

    ch1.register_receiving(t)

    assert ch1.has_pair()


def test_channel_clean_up():
    """
    Test the channel clean up function.
    """
    ch = Channel(False)
    n = ID("name")
    g = Guard(n)
    u = Update([n])
    s = Synchronization(False, n)
    i = Invariant(n)

    t1 = Transition(g, u, s)
    t2 = Transition(g, u, s)

    l1 = Location("id0", "location1", i)
    l2 = Location("id1", "location2", i)
    l3 = Location("id2", "location3")
    t1.source = l1
    t1.target = l3
    t2.source = l2
    t2.target = l3
    l1.add_transition(t1)
    l2.add_transition(t2)

    ch.register_sending(t1)
    ch.register_sending(t2)
    ch.register_receiving(t1)
    ch.register_receiving(t2)

    assert len(ch.receiving_transitions) == 2
    assert len(ch.sending_transitions) == 2

    ch.clean_up(l1)

    assert len(ch.receiving_transitions) == 1
    assert len(ch.sending_transitions) == 1

    ch.clean_up(l2)

    assert len(ch.receiving_transitions) == 0
    assert len(ch.sending_transitions) == 0


#def test_clock():
#    c = Clock()
#    t = time.time()
#    while time.time() - t < 1.0:
#        assert (c.value >= (time.time() - t) * 0.99)
#        assert (c.value <= (time.time() - t) * 1.01)
#        time.sleep(0.01)


def test_range_at_lower_limit():
    """
    Test the range at the lower limit.
    """
    r = Range(False, 2, 4, 3)
    try:
        r.value = 2
    except BelowRangeError:
        assert False
    with pytest.raises(BelowRangeError):
        r.value = 1


def test_range_at_upper_limit():
    """
    Test the range at the upper limit.
    """
    r = Range(False, 2, 4, 3)
    try:
        r.value = 4
    except ExceedsRangeError:
        assert False
    with pytest.raises(ExceedsRangeError):
        r.value = 5


def test_change_constant_values():
    """
    Test if changing of a constant value raises an error.
    """
    i = IntVariable(True, 34)
    with pytest.raises(ImmutableValueError):
        i.value = 4
